#include <cassert>

#include <iostream>
using std::cout;
using std::endl;

#include <iomanip>
using std::setw;

#include <cmath>
using std::cos;
using std::exp;

double trapezoidIntegrator( double (&f)(double), double a, double b, unsigned int N)
{
	assert(N > 0);
	const double width = b-a;
	const double h = width/N;
	double sum = 0.5*( f(a) + f(b) );
	for (unsigned int i = 1; i < N; ++i)
	{
		const double x_i = a + i*h;
		sum += f(x_i);
	}
	return sum*h;
}

double SimpsonIntegrator( double (&f)(double), double a, double b, unsigned int N_)
{
	const unsigned int N = ( N_%2 == 1 ? N_+1 : N_ );
	assert(N%2 == 0);
	assert(N > 1);
	
	const double width = b-a;
	const double h = width/N;
	double sum = f(a) + 4*f(a+h) + f(b);
	for (unsigned int i = 2; i < N; i += 2)
	{
		const double x_i = a + i*h;
		sum += 2*f(x_i) + 4*f(x_i+h);
	}
	return sum*h/3.0;
}

double myFunc(double x)
{
	const double y = cos(M_PI*x);
	return y*y*x*exp(x);
}

int main()
{
	cout.precision(13);
	for (unsigned int N = 10; N <= 2560; N *= 2)
	{
		const double step_size = 2.0/N;
		cout << setw(16) << N << setw(16) << step_size << setw(20) << trapezoidIntegrator(myFunc,0.0,2.5,N) << endl;
	}
	
	cout << endl << endl;
	
	for (unsigned int N = 10; N <= 2560; N *= 2)
	{
		const double step_size = 2.0/N;
		cout << setw(16) << N << setw(16) << step_size << setw(20) << SimpsonIntegrator(myFunc,0.0,2.5,N) << endl;
	}
	
	return 0;
}
