#include <cassert>

#include <cstdlib>
using std::atoi;

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <iomanip>
using std::setw;

#include "particle.hpp"

// acceleration model:

const double K = 1.0;

double HookesLaw(const particle &p)
{
	return -(K/p.mass)*p.x;
}

// integrator method:

void Euler(particle &p, double a, double dt)
{
	const double v_old = p.v;
	p.v += a*dt;
	p.x += v_old*dt;
	p.t += dt;
}

void EulerCromer(particle &p, double a, double dt)
{
	// insert code
}

void Verlet(particle &p, double (&accel)(const particle&), double dt)
{
	// insert code
}

void RungaKutta(particle &p, double (&accel)(const particle&), double dt)
{
	// insert code
}

int main(int argc, char *argv[])
{
	assert(argc == 2);

	const double duration = 100.0; // run simulation for 100 s
	const unsigned long int steps = atoi(argv[1]); // read numer of steps from command line
	const double dt = duration / steps;
	cerr << "Simulating harmonic oscillator with time step dt = " << dt << endl;
	
	particle p(1.0,3.0,0.0); 

	for (unsigned long int n = 0; n < steps; ++n)
	{
		double energy = 0.5*p.mass*p.v*p.v + 0.5*K*p.x*p.x; 
		cout << p << setw(20) << energy << endl; // output the particle information in (t,x,v,E) format
		p.evolve(HookesLaw,Euler,dt); // perform a single Euler update for Hooke's law
	}

	return 0;
}