#include <cassert>

#include <cstddef>
using std::size_t;

#include <cstdlib>
using std::atoi;

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <fstream>
using std::ofstream;

#include <iomanip>
using std::setw;

#include <cmath>

#include <vector>
using std::vector;

#include "particle.hpp"

// acceleration model:

const double K = 1.0;

double HookesLaw(const particle &p)
{
	return -(K/p.mass)*p.x;
}

double pendulum(const particle &p)
{
	// insert your code
}

// integrator method:

void Euler(particle &p, double a, double dt)
{
	const double v_old = p.v;
	p.v += a*dt;
	p.x += v_old*dt;
	p.t += dt;
}

void EulerCromer(particle &p, double a, double dt)
{
	// insert code
}

void Verlet(particle &p, double (&accel)(const particle&), double dt)
{
	// insert code
}

void RungaKutta(particle &p, double (&accel)(const particle&), double dt)
{
	// insert code
}

int main(int argc, char *argv[])
{
	assert(argc == 2);

	const double duration = 100.0; // run simulation for 100 s
	const unsigned int steps = 10000; 
	const double dt = duration / steps; // dt = 0.01;
	const unsigned int print_interval = 10;
	const size_t num_pendula = atoi(argv[1]); // read number of pendula from command line

	cerr << "Simulating " << num_pendula << " pendula with time step dt = " << dt << endl;
	
	vector<particle> p; // declare an empty (size 0) vector of particles
	for (size_t k = 0; k < num_pendula; ++k)
	{
		const double theta = 2.0*M_PI*k/num_pendula;
		double R = 3.0+2.0*cos(4.0*theta);
		double x = R*cos(theta);
		double v = R*sin(theta);
		p.push_back( particle(1.0,x,v) ); // add a particle to the end of the vector p
	}
	assert( p.size() == num_pendula );
	
	ofstream fout("pendula.dat");
	for (unsigned long int n = 0; n < steps; ++n)
	{
		if (n%print_interval == 0)
		{
			for (size_t k = 0; k < num_pendula; ++k)
				fout << p[k] << endl;
			fout << p[0] << endl;
			fout << endl << endl;
		}
		for (size_t k = 0; k < num_pendula; ++k)
		{
			p[k].evolve(HookesLaw,Euler,dt);
			//p[k].evolve(pendulum,EulerCromer,dt);
			//p[k].evolve(pendulum,Verlet,dt);
			//p[k].evolve(pendulum,RungaKutta,dt);
		}
	}
	fout.close();

	return 0;
}